#!/usr/bin/python
#
# Copyright (C) 2006 Jesus Espino Garcia <jespino@di.uc3m.es>
#
# Based on metadistros project of Juan Jesus Ojeda <juanje@interactors.coop>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os
import shutil
import re
import sys
import time

class Gendist:
	def __init__(self):
		self.__iso_name = self.__get_config("ISO_NAME")

		if self.__iso_name == "":
			today = str(time.localtime()[0])+"-"+str(time.localtime()[1])+"-"+str(time.localtime()[2])
			self.__iso_name = "cd_distribution_"+today+".iso"

		# Loading default distribution
		self.__default_dist = self.__set_default_dist()

		# Loading Distributions
		self.__distributions = []
		if os.path.isfile("gendist.config") and os.path.isdir("./dists"):
			for dir in os.listdir("./dists/"):
				if os.path.isdir("./dists/"+dir):
					self.__distributions.append(Distribution(dir))
		# Loading Boot
		self.__boot_type = self.__get_config("BOOT_TYPE")
		if self.__boot_type == "gfxboot":
			boot_artwork = self.__get_config("GFXBOOT_THEME")
			bootlogo = self.__get_config("ISOLINUXLOGO")
			self.__boot = GfxBoot(boot_artwork,bootlogo,self.__distributions,self.__default_dist)
		elif self.__boot_type == "grub":
			boot_artwork = self.__get_config("GRUBLOGO")
			self.__boot = GrubBoot(boot_artwork,self.__distributions,self.__default_dist)
		elif self.__boot_type == "isolinux":
			boot_artwork = self.__get_config("ISOLINUXLOGO")
			self.__boot = IsolinuxBoot(boot_artwork,self.__distributions,self.__default_dist)

	def __get_config(self,parameter):
		config_file = file("gendist.config","r")
		value = ""
		for line in config_file:
			pair = re.match("(^.*)=[\"\'](.*)[\"\']",line)
			if pair:
				if pair.group(1)==parameter:
					value = pair.group(2)
		return value

	def get_default_dist(self):
		return self.__default_dist

	def get_dists(self):
		return self.__distributions

	def __set_default_dist(self):
		default_dist=""
		if os.path.isfile("gendist.config") and os.path.exists("dists"):
			dist = self.__get_config("DEFAULT_DIST")
			if dist=="":
				default_dist = os.listdir("dists/")[0]
			else:
				default_dist = dist
		return default_dist

	def __check_boot(self):
		if os.path.exists("master/boot/isolinux/isolinux.bin") or os.path.exists("master/boot/grub/menu.lst"):
			return 0
		else:
			print "Error: The boot system not exists."
			print "       run 'gendist mk_boot' before."
			sys.exit(1)

	def __check_dist(self):
		if len(self.__distributions)==0:
			print "Error: Not exists any distribution."
			print "       run 'gendist add_dist <dist_name>' before."
			sys.exit(1)
		else:
			return 0

	def generate_boot(self):
		self.__boot.generate()

	def generate_dists(self):
		self.__check_dist()
		for dist in self.__distributions:
			dist.generate()

	def generate_iso(self):
		self.__check_boot()
		if not os.path.exists("iso"):
			os.mkdir("iso")
        	if self.__boot_type == "gfxboot" or self.__boot_type == "isolinux":
			if os.path.exists("master/boot/isolinux/isolinux.bin"):
        			os.system("mkisofs -f -l -r -J -V "+self.__default_dist+" -hide-rr-moved -v -b boot/isolinux/isolinux.bin -no-emul-boot -boot-load-size 4 -boot-info-table -o iso/"+self.__iso_name +" master/")
			else:
				print "Error: Boot not generated."
        	elif self.__boot_type == "grub":
			if os.path.exists("master/boot/grub/stage2_eltorito"):
        			os.system("mkisofs -f -l -r -J -V "+self.__default_dist+" -hide-rr-moved -v -b boot/grub/stage2_eltorito -no-emul-boot -boot-load-size 4 -boot-info-table -o iso/"+self.__iso_name +" master/")
			else:
				print "Error: Boot not generated."

	def generate_usb_generator(self):
		'''Generate a file for usb devices'''
		print "Method not implemented"

	def get_boot(self):
		return self.__boot

	def get_distribution(self,dist_name):
		self.__check_dist()
		return_dist = None
		for dist in self.__distributions:
			if dist.get_name() == dist_name:
				return_dist = dist
		return return_dist

	def get_distributions_list(self):
		self.__check_dist()
		return self.__distributions


class Boot:
	def __init__(self,artwork,dists):
		self.__artwork = artwork
		self.__dists = dists

	def generate(self):
		self.generate_menu()
		self.generate_artwork()
		if not os.path.exists("master/boot/memtest") and os.path.exists("/boot/memtest86+.bin"):
			os.symlink("/boot/memtest86+.bin","master/boot/memtest")

class GfxBoot(Boot):
	def __init__(self,artwork,bootlogo,dists,default_dist):
		self.__artwork = artwork
		self.__bootlogo = bootlogo
		self.__dists = dists
		self.__default_dist = default_dist

	def generate_menu(self):
		if os.path.exists("master/boot/isolinux"):
			shutil.rmtree("master/boot/isolinux")
		if os.path.exists("master/boot/grub"):
			shutil.rmtree("master/boot/grub")
		shutil.copytree("/usr/share/gendist/master/boot/isolinux","master/boot/isolinux")
		os.symlink("/usr/lib/syslinux/isolinux.bin","master/boot/isolinux/isolinux.bin")

		isolinux_txt_file = file("master/boot/isolinux/isolinux.txt","w")
		isolinux_txt_file.write("splash.rle\n")
		isolinux_txt_file.close()

		isolinux_cfg_file = file("master/boot/isolinux/isolinux.cfg","w")

		default = self.__default_dist+"_"
		for dist in self.__dists:
			if dist.get_name() == self.__default_dist:
				default = default+dist.get_default_lang()
				
		isolinux_cfg_file.write("default "+default.split(":")[0]+"\n")
		isolinux_cfg_file.write("DISPLAY isolinux.txt\n")
		isolinux_cfg_file.write("gfxboot init\n")
		isolinux_cfg_file.write("TIMEOUT 100\n")
		isolinux_cfg_file.write("PROMPT 1\n")
		isolinux_cfg_file.write("KBDMAP es.kbd\n")
		isolinux_cfg_file.write("FONT fuente.psf\n")

		if os.path.exists("master/boot/memtest"):
			isolinux_cfg_file.write("\nlabel memtest\n")
  			isolinux_cfg_file.write("kernel /boot/memtest\n")

		languages_file = file("master/boot/isolinux/languages","w")
		for dist in self.__dists:
			for lang in dist.get_langs():
				isolinux_cfg_file.write("\nlabel "+dist.get_name()+"_"+lang.split(":")[0]+"\n")
				isolinux_cfg_file.write("kernel /boot/vmlinuz."+dist.get_name()+"\n")
				isolinux_cfg_file.write("append ramdisk_size=100000 splash root=/dev/ram0 initrd=/boot/initramfs."+dist.get_name()+" BOOT=live vga=791 debug quiet lang="+lang.split(":")[1]+"\n")
				languages_file.write(lang.split(":")[1]+"\n")

	def generate_artwork(self):
		shutil.copytree(self.__artwork,"/tmp/gfxboot_theme_tmp")
		os.system("make -s -C /tmp/gfxboot_theme_tmp 2> /dev/null")
		for gfxboot_theme_file in os.listdir("/tmp/gfxboot_theme_tmp/boot/"):
			shutil.copy("/tmp/gfxboot_theme_tmp/boot/"+gfxboot_theme_file,"master/boot/isolinux/"+gfxboot_theme_file)
		shutil.rmtree("/tmp/gfxboot_theme_tmp")

		os.system("convert -scale x300 -colors 16 \""+self.__bootlogo+"\" /tmp/bootlogo.bmp")
        	os.system("bmptoppm < /tmp/bootlogo.bmp 2> /dev/null | ppmtolss16 > \"master/boot/isolinux/splash.rle\" 2> /dev/null")
        	os.remove("/tmp/bootlogo.bmp")

class GrubBoot(Boot):
	def __init__(self,artwork,dists,default_dist):
		self.__artwork = artwork
		self.__dists = dists
		self.__default_dist = default_dist

	def generate_menu(self):
		if os.path.exists("master/boot/isolinux"):
			shutil.rmtree("master/boot/isolinux")
		if os.path.exists("master/boot/grub"):
			shutil.rmtree("master/boot/grub")
		os.makedirs("master/boot/grub")

		system_arch = os.uname()[4]
		if re.match("i[0-9]86|ia32|ia64",system_arch):
			deb_arch="i386"
		else:
			deb_arch=system_arch

		os.symlink("/lib/grub/"+deb_arch+"-pc/stage2_eltorito","master/boot/grub/stage2_eltorito")

		menu_lst_file = file("master/boot/grub/menu.lst","w")
		menu_lst_file.write("timeout 30\n")
		menu_lst_file.write("fallback 1\n")

		# Writing default dist
		if os.path.exists("master/boot/memtest"):
			counter = 0
		else:
			counter = -1
		default = 0
		for dist in self.__dists:
			for lang in dist.get_langs():
				counter += 1
				if dist.get_name() == self.__default_dist and lang==dist.get_default_lang():
					default = counter
		menu_lst_file.write("\ndefault "+str(default)+"\n")

		# Writing artwork lines
		menu_lst_file.write("splashimage=/boot/grub/grublogo.xpm.gz\n")
		menu_lst_file.write("background=800000\n")

		#menu_lst_file.write("title		Developer Mode\n")
		#menu_lst_file.write("kernel		/boot/vmlinuz ramdisk_size=100000 root=/dev/ram0 BOOT=live debug\n")
		#menu_lst_file.write("initrd		/boot/initramfs\n")
		#menu_lst_file.write("boot\n")

		if os.path.exists("master/boot/memtest"):
			# Writing memory test lines
			menu_lst_file.write("title		Memory Test\n")
			menu_lst_file.write("kernel		/boot/memtest\n")
			menu_lst_file.write("boot\n")

		# Writing distributions entries
		for dist in self.__dists:
			for lang in dist.get_langs():
				menu_lst_file.write("\ntitle "+dist.get_name()+" "+lang.split(":")[0]+"\n")
				menu_lst_file.write("kernel /boot/vmlinuz."+dist.get_name()+" ramdisk_size=100000 splash root=/dev/ram0 BOOT=live vga=791 debug quiet lang="+lang.split(":")[1]+"\n")
				menu_lst_file.write("initrd /boot/initramfs."+dist.get_name()+"\n")
				menu_lst_file.write("boot\n")

		menu_lst_file.close()

	def generate_artwork(self):
        	os.system("convert -scale 640x480 -colors 14 "+self.__artwork+" /tmp/grublogo.xpm")
        	os.system("gzip -f /tmp/grublogo.xpm")
		shutil.move("/tmp/grublogo.xpm.gz","master/boot/grub/grublogo.xpm.gz")

class IsolinuxBoot(Boot):
	def __init__(self,artwork,dists,default_dist):
		self.__artwork = artwork
		self.__dists = dists
		self.__default_dist = default_dist

	def generate_menu(self):
		if os.path.exists("master/boot/isolinux"):
			shutil.rmtree("master/boot/isolinux")
		if os.path.exists("master/boot/grub"):
			shutil.rmtree("master/boot/grub")
		shutil.copytree("/usr/share/gendist/master/boot/isolinux","master/boot/isolinux")
		os.symlink("/usr/lib/syslinux/isolinux.bin","master/boot/isolinux/isolinux.bin")

		isolinux_txt_file = file("master/boot/isolinux/isolinux.txt","w")
		isolinux_txt_file.write("splash.rle")
		isolinux_txt_file.write("\n\n\n\n\n\n")
		isolinux_txt_file.write("Welcome, push ENTER to continue.\n")
		isolinux_txt_file.write("To view help, push F2.\n")
		isolinux_txt_file.close()

		f2_txt_file = file("master/boot/isolinux/f2.txt","w")
		f2_txt_file.write("0fHELP07                                                                  09F207\n")
		f2_txt_file.write("\nYou must have at least 64MB of RAM memmory.\n")
		f2_txt_file.write("\nFor more information use the distribution documentation.\n")
		f2_txt_file.write("\nOptions:                                          \n")
		for dist in self.__dists:
			f2_txt_file.write("  "+dist.get_name()+" - Load the "+dist.get_name()+" distribution.\n")
		if os.path.exists("master/boot/memtest"):
			f2_txt_file.write("  memtest - Load the memmory test.                \n")
		f2_txt_file.write("\nParameters:                                     \n")
		f2_txt_file.write("  lang=<country-code> - Select your language (Ex: linux lang=en).\n")
		supported_langs = "\nSupporte langs:"
		for dist in self.__dists:
			for lang in dist.get_langs():
				supported_langs = supported_langs+" "+lang.split(":")[1]
		f2_txt_file.write(supported_langs+"\n")
		f2_txt_file.write("\nPush F1control & F then 1 to return to main screen, or ENTER to continue.\n")
		f2_txt_file.close()

		isolinux_cfg_file = file("master/boot/isolinux/isolinux.cfg","w")
		isolinux_cfg_file.write("default linux\n")
		isolinux_cfg_file.write("DISPLAY isolinux.txt\n")
		isolinux_cfg_file.write("TIMEOUT 100\n")
		isolinux_cfg_file.write("PROMPT 1\n")
		isolinux_cfg_file.write("F1 isolinux.txt\n")
		isolinux_cfg_file.write("F2 f2.txt\n")
		isolinux_cfg_file.write("KBDMAP es.kbd\n")
		isolinux_cfg_file.write("FONT fuente.psf\n")
		for dist in self.__dists:
			isolinux_cfg_file.write("\nlabel "+dist.get_name()+"\n")
			isolinux_cfg_file.write("  kernel /boot/vmlinuz."+dist.get_name()+"\n")
			isolinux_cfg_file.write("  append ramdisk_size=100000 splash root=/dev/ram0 initrd=/boot/initramfs."+dist.get_name()+" BOOT=live vga=791 debug quiet\n")
		if os.path.exists("master/boot/memtest"):
			isolinux_cfg_file.write("\nlabel memtest\n")
			isolinux_cfg_file.write("  kernel /boot/memtest\n")
		isolinux_cfg_file.close()

	def generate_artwork(self):
		os.system("convert -scale x300 -colors 16 \""+self.__artwork+"\" /tmp/bootlogo.bmp")
        	os.system("bmptoppm < /tmp/bootlogo.bmp 2> /dev/null | ppmtolss16 > \"master/boot/isolinux/splash.rle\" 2> /dev/null")
        	os.remove("/tmp/bootlogo.bmp")


class Distribution:
	def __init__(self,name):
		# Distribution Config
		self.__name = name
		if len(self.__get_config("VERSION"))>0:
			self.__version = self.__get_config("VERSION")[0]
		else:
			self.__version = "0.1"

		# Initramfs Config
		if len(self.__get_config("KERNEL_VERSION"))>0:
			kernel_version = self.__get_config("KERNEL_VERSION")[0]
		else:
			kernel_version = ""

		if kernel_version == "":
			kernel_version = os.uname()[2]

		if len(self.__get_config("SPLASH_LOGO"))>0:
			splash_logo = self.__get_config("SPLASH_LOGO")[0]
		else:
			splash_logo = "/usr/share/gendist/artwork/gendist-splash.png"

		mods_and_parms = self.__get_config("MODS_AND_PARMS")

		# Image Config
		dns_server = self.__get_config("DNS_SERVER")
		hostname = self.__get_config("HOSTNAME")[0]
		repositories = self.__get_config("REPOSITORIES")

		if len(self.__get_config("DEB_VERSION"))>0:
			deb_version = self.__get_config("DEB_VERSION")[0]
		else:
			deb_version = "dapper"

		if len(self.__get_config("BASE_MIRROR"))>0:
			base_mirror = self.__get_config("BASE_MIRROR")[0]
		else:
			base_mirror = "http://ftp.ubuntu.com"

		packages = self.__get_config("PACKAGES")
		packages.append("linux-image-"+kernel_version)

		# Documentation Config
		if len(self.__get_config("DOC_DIRECTORY"))>0:
			doc_directory = self.__get_config("DOC_DIRECTORY")[0]
		else:
			doc_directory = ""

		self.__image = Image(dns_server,hostname,repositories,deb_version,base_mirror,packages,self.__name,self.__version)
		self.__initramfs = Initramfs(kernel_version,splash_logo,mods_and_parms,self.__name)
		self.__documentation = Documentation(doc_directory,self.__name)

	def __get_config(self,parameter):
		config_file = file("dists/"+self.__name+"/dist.config","r")
		value = []
		for line in config_file:
			pair = re.match("(^.*)=\"(.*)\"",line)
			if pair:
				if pair.group(1)==parameter:
					value.append(pair.group(2))
		return value

	def generate(self):
		print "Generating Image:"
		self.__image.generate()
		print "Generating Initramfs:"
		self.__initramfs.generate()
		print "Generating Documentation:"
		self.__documentation.generate()

	def generate_image(self):
		self.__image.generate()

	def generate_initramfs(self):
		self.__initramfs.generate()

	def generate_documentation(self):
		self.__documentation.generate()

	def get_image(self):
		return self.__image

	def get_initramfs(self):
		return self.__initramfs

	def get_documentation(self):
		return self.__documentation

	def get_default_lang(self):
		return self.__initramfs.get_langs()[0]

	def get_langs(self):
		return self.__initramfs.get_langs()

	def get_name(self):
		return self.__name

class Image:
	def __init__(self,dns_server,hostname,repositories,deb_version,base_mirror,packages,dist_name,dist_version):
		self.__dns_server = dns_server
		self.__hostname = hostname 
		self.__repositories = repositories 
		self.__deb_version = deb_version 
		self.__dist_version = dist_version 
		self.__base_mirror = base_mirror 
		self.__dist_name = dist_name
		self.__image_dir = "./dists/"+dist_name+"/image"
		self.__arch = self.__detect_arch()
		self.__packages = packages

	def __check_base(self):
		if os.path.exists(self.__image_dir+"/bin") \
	 	   and os.path.exists(self.__image_dir+"/sbin") \
	 	   and os.path.exists(self.__image_dir+"/usr/bin") \
	 	   and os.path.exists(self.__image_dir+"/usr/sbin") \
	 	   and os.path.exists(self.__image_dir+"/tmp") \
	 	   and os.path.exists(self.__image_dir+"/lib") \
	 	   and os.path.exists(self.__image_dir+"/var"):
			return 0
		else:
			print "Error: The base system for "+self.__dist_name+" not exists."
			print "       run 'gendist "+self.__dist_name+" source' before."
			sys.exit(1)

	def configure(self):
		# Issue config
		issue_file = file(self.__image_dir+"/etc/issue","w")
		issue_file.write(self.__dist_name+" "+self.__dist_version+" \\n \\l\n")
		issue_file.close()
		issue_net_file = file(self.__image_dir+"/etc/issue.net","w")
		issue_net_file.write(self.__dist_name+" "+self.__dist_version+"\n")
		issue_net_file.close()

		# DNS Config
		resolv_conf_file = file(self.__image_dir+"/etc/resolv.conf","w")
		for dns_server in self.__dns_server:
			resolv_conf_file.write("nameserver "+dns_server+"\n")
		resolv_conf_file.close()

		# Hostname Config
		hostname_file = file(self.__image_dir+"/etc/hostname","w")
		hostname_file.write(self.__hostname+"\n")
		hostname_file.close()

		sources_list_file = file(self.__image_dir+"/etc/apt/sources.list","w")
		for repository in self.__repositories:
			sources_list_file.write(repository+"\n")
		sources_list_file.close()
	
	def generate(self):
		print "- Generating base system:"
		self.generate_base()
		print "- Installing software:"
		self.install_package(self.__packages)
		print "- Cleaning cache:"
		self.clean_cache()
		print "- Building Meta File:"
		self.build_meta()

	def generate_base(self):
		if os.path.exists(self.__image_dir):
			print "Warning: directory "+self.__image_dir+" exists. Not executing base."
		else:
			os.system("debootstrap --arch "+self.__arch+" "+self.__deb_version+" "+self.__image_dir+" "+self.__base_mirror)
			self.configure()

	def run_command(self,command):
		self.__check_base()
		self.__chroot_command(" ".join(command))

	def sim_install_package(self,packages):
		self.__check_base()
		self.configure()
		self.__chroot_command("apt-get update")
		command_line = "apt-get install --simulate"
		for package in packages:
			command_line = command_line+" "+package
		self.__chroot_command(command_line)

	def install_package(self,packages):
		self.__check_base()
		self.configure()
		self.__chroot_command("apt-get update")
		command_line = "apt-get install --force-yes -y"
		for package in packages:
			command_line = command_line+" "+package
		self.__chroot_command(command_line)
		self.configure()

	def install_conf_packages(self):
		self.__check_base()
		self.install_package(self.__packages)
		
	def install_deb_files(self,deb_files_paths):
		self.__check_base()
		command_line = "dpkg -i"
		for deb_file in deb_files_paths:
			shutil.copy(deb_file,self.__image_dir)
			command_line = command_line+" "+os.path.basename(deb_file)
		self.__chroot_command(command_line)
		self.__chroot_command("apt-get install --force-yes -y -f")
		os.remove(self.__image_dir+"/"+os.path.basename(deb_file))

	def clean_cache(self):
		self.__check_base()
		self.__chroot_command("apt-get clean")

	def build_meta(self):
		self.__check_base()
		if not os.path.exists("master/META"):
			os.makedirs("master/META")
		if os.path.exists("master/META/"+self.__dist_name+".squashfs"):
			os.remove("master/META/"+self.__dist_name+".squashfs")
		os.system("mksquashfs "+self.__image_dir+" master/META/"+self.__dist_name+".squashfs")

	def __chroot_command(self,command):
		os.system("chroot \""+self.__image_dir+"\" mount /proc 2> /dev/null")
		ok = os.system("chroot \""+self.__image_dir+"\" "+command)
		os.system("chroot \""+self.__image_dir+"\" umount /proc/sys/fs/bin_fmt 2> /dev/null")
		os.system("chroot \""+self.__image_dir+"\" umount /proc 2> /dev/null")
		return ok

	def __detect_arch(self):
		system_arch = os.uname()[4]
		if re.match("i[0-9]86|ia32|ia64",system_arch):
			deb_arch="i386"
		elif system_arch == "x86_64":
			deb_arch="amd64"
		elif re.match("sparc*",system_arch):
			deb_arch="sparc"
		else:
			deb_arch=system_arch
		return deb_arch

class Documentation:
	def __init__(self,documentation_dir,dist_name):
		self.__directory = documentation_dir
		self.__dist_name = dist_name

	def generate(self):
		if self.__directory != "" and os.path.exists(self.__directory):
			os.symlink(self.__directory,"master/doc_"+self.dist_name)

class Initramfs:
	def __init__(self,kernel_version,splash_logo,mods_and_parms,dist_name):
		self.__kernel_version = kernel_version
		self.__splash_logo = splash_logo
		self.__dist_name = dist_name
		self.__modules = []
		self.__langs = []
		for module in mods_and_parms:
			mod_name=module.split(',')[0]
			mod_parameters=module.replace(mod_name+",","")
			self.__modules.append(InitramfsModule(mod_name,mod_parameters,self.__dist_name))
			if mod_name == "lang":
				default_lang_param = mod_parameters.split(",")[0]
				other_langs_param = mod_parameters.split(",")[1]
				self.__langs.append(re.match("^.*=(.*)",default_lang_param).group(1))
				for other_lang in re.match("^.*=(.*)",other_langs_param).group(1).split(" "):
					if other_lang != "":
						self.__langs.append(other_lang)
		if self.__langs == []:
			self.__langs.append("English:en_EN")

	def __check_base(self):
		if os.path.exists("dists/"+self.__dist_name+"/initramfs/scripts"): 
			return 0
		else:
			print "Error: The initramfs base system for "+self.__dist_name+" not exists."
			print "       run 'gendist "+self.__dist_name+" mk_initramfs' before."
			sys.exit(1)
		
	def generate(self):
		print "- Generating Initramfs base:"
		self.generate_base()
		print "- Building Initramfs file:"
		self.build_initramfs()

	def get_langs(self):
		return self.__langs

	def __install_modules(self):
		for module in self.__modules:
			module.install()

	def add_module(self,module_name,module_parms):
		self.__modules.append(InitramfsModule(module_name,module_parms,self.__dist_name))

	def __add_splash(self):
		# Adapt the image to usplash characteristics
		os.system("convert -scale 640x480 -colors 16 \""+self.__splash_logo+"\" usplash-artwork.png")
	
		# Convert the image to usplash-artwork.so format
		os.system("pngtobogl usplash-artwork.png > usplash-artwork.c")
		os.system("gcc -Os -g -I/usr/include/bogl -fPIC -c usplash-artwork.c -o usplash-artwork.o")
		os.system("gcc -shared -Wl usplash-artwork.o -o usplash-artwork.so")
	
		if not os.path.exists("dists/"+self.__dist_name+"/initramfs/usr/lib/usplash/"):
			os.makedirs("dists/"+self.__dist_name+"/initramfs/usr/lib/usplash/")

		# Move the image.so to initramfs
		shutil.move("usplash-artwork.so","dists/"+self.__dist_name+"/initramfs/usr/lib/usplash/usplash-artwork.so")
	
		# Remove the temporal files
		os.remove("usplash-artwork.png")
		os.remove("usplash-artwork.c")
		os.remove("usplash-artwork.o")

	def generate_base(self):
		if not os.path.isdir("/lib/modules/"+self.__kernel_version):
			print "Error: The kernel "+self.__kernel_version+" not installed in your system."
		else:
			if not os.path.isdir("master/boot"):
				os.makedirs("master/boot")
			if os.path.exists("master/boot/vmlinuz."+self.__dist_name):
				os.remove("master/boot/vmlinuz."+self.__dist_name)
			os.symlink("/boot/vmlinuz-"+self.__kernel_version,"master/boot/vmlinuz."+self.__dist_name)
			os.system("mkinitramfs -d /usr/share/gendist/mkinitramfs -o /tmp/initramfs_gendist_tmp "+self.__kernel_version)
			if not os.path.exists("dists/"+self.__dist_name+"/initramfs"):
				os.mkdir("dists/"+self.__dist_name+"/initramfs")
			self.__import_initramfs("/tmp/initramfs_gendist_tmp","dists/"+self.__dist_name+"/initramfs")
			os.remove("/tmp/initramfs_gendist_tmp")
			self.__install_modules()
			self.__add_splash()
			self.__apply_dist_name_premount()

	def __apply_dist_name_premount(self):
		local_file = file("dists/"+self.__dist_name+"/initramfs/scripts/live-premount/local")
		local_file_string = local_file.read()
		local_file_string = local_file_string.replace("__DIST_NAME__",self.__dist_name)
		local_file.close()
		local_file = file("dists/"+self.__dist_name+"/initramfs/scripts/live-premount/local","w")
		local_file.write(local_file_string)
		local_file.close()

		setup_image_file = file("dists/"+self.__dist_name+"/initramfs/scripts/live-premount/setup_image")
		setup_image_file_string = setup_image_file.read()
		setup_image_file_string = setup_image_file_string.replace("__DIST_NAME__",self.__dist_name)
		setup_image_file.close()
		setup_image_file = file("dists/"+self.__dist_name+"/initramfs/scripts/live-premount/setup_image","w")
		setup_image_file.write(setup_image_file_string)
		setup_image_file.close()

	def __import_initramfs(self,src,dst):
		shutil.copy(src,"/tmp/initramfs.gz")
		if os.path.exists("/tmp/initramfs"):
			os.remove("/tmp/initramfs")
		os.system("gunzip /tmp/initramfs.gz")
		shutil.rmtree(dst)
		os.makedirs(dst)
		dir=os.getcwd()
		os.chdir(dst)
		os.system("cpio -i < /tmp/initramfs 2> /dev/null")
		os.remove("/tmp/initramfs")
		os.chdir(dir)

	def build_initramfs(self):
		self.__check_base()
		if not os.path.exists("master/boot"):
		    os.makedirs("master/boot")
		os.chdir("dists/"+self.__dist_name+"/initramfs")
		os.system("find -iname '*' | cpio -o -L -H newc > ../../../initramfs 2> /dev/null")
		os.chdir("../../..")
		os.system("gzip initramfs")
		shutil.move("initramfs.gz","master/boot/initramfs."+self.__dist_name)

	def remove_module(self,module_name):
		for module in self.__modules:
			if module.get_name() == module_name:
				del(module)

	def get_modules_list(self):
		return self.__modules

class InitramfsModule:
	def __init__(self,name,params,dist):
		self.__name = name
		self.__params = []
		self.__dist = dist
		params_list = params.split(",")
		for param in params_list:
			param_name=param.split('=')[0]
			if len(param.split('=')) == 2:
				param_value=param.split('=')[1]
			else:
				param_value=''
			self.__params.append((param_name,param_value))

	def install(self):
		module_src_file = file("/usr/share/gendist/initramfs_modules/"+self.__name)
		module_string = module_src_file.read()
		module_src_file.close()
		module_string = self.__apply_parameters(module_string)
		module_dst_file = file("dists/"+self.__dist+"/initramfs/scripts/live-bottom/"+self.__name,"w")
		module_dst_file.write(module_string)
		os.chmod("dists/"+self.__dist+"/initramfs/scripts/live-bottom/"+self.__name,755)
		module_dst_file.close()

	def __apply_parameters(self,script_text):
		for param in self.__params:
			script_text = script_text.replace("__"+param[0]+"__",param[1])
		return script_text

	def get_name(self):
		return self.__name
